﻿using static AsReaderA24DMAUIDemo.Platforms.Droid.AsReaderA24DDefine;
using AsReaderA24DSDK;
using JP.CO.Asterisk.Asreader.A24d.Sdk;
using JP.CO.Asterisk.Asreader.Barcodemanager;
using JP.CO.Asterisk.Asreader.Communicationmanager.Connection;
using JP.CO.Asterisk.Asreader.Devicemanager;
using JP.CO.Asterisk.Asreader.Utility;
using JP.CO.Asterisk.Asreader.Barcodemanager.Define;

namespace AsReaderA24DMAUIDemo.Platforms.Droid
{
    public class AsReaderA24DManager
    {
        public class BarcodeAsReaderDeviceDelegate : Java.Lang.Object, IBarcodeManagerCallback
        {
            AsReaderA24DDelegate asReaderA24DDelegate = AsReaderA24DDelegate.sharedInstance();
            public BarcodeAsReaderDeviceDelegate()
            {

            }

            public void OnReceivedBarcodeDecodeData(string decodeData, IDictionary<string, string> parameter)
            {
                if (asReaderA24DDelegate.OnReceivedBarcodeDecodeData != null)
                {
                    Dictionary<string, string> dic = new Dictionary<string, string>();
                    foreach (string key in parameter.Keys)
                    {
                        dic[key] = parameter[key];  
                    }
                    asReaderA24DDelegate.OnReceivedBarcodeDecodeData(decodeData, dic);
                }
            }

            public void OnReceivedBarcodeDecodeData(byte[] decodeData)
            {
                if (asReaderA24DDelegate.OnReceivedBarcodeDecodeByteData != null)
                {
                    asReaderA24DDelegate.OnReceivedBarcodeDecodeByteData(decodeData);
                }
            }

            public void OnBarcodeManagerError(BarcodeManagerError errorCode)
            {
                if (asReaderA24DDelegate.OnBarcodeManagerError != null)
                {
                    AsReaderA24D_BarcodeManagerError code = (AsReaderA24D_BarcodeManagerError)Convert.ToInt32(errorCode);
                    asReaderA24DDelegate.OnBarcodeManagerError(code);
                }
            }
        }
        public class MyAsReaderDeviceDelegate : Java.Lang.Object, IDeviceManagerCallback
        {
            AsReaderA24DDelegate asReaderA24DDelegate = AsReaderA24DDelegate.sharedInstance();
            public MyAsReaderDeviceDelegate()
            {

            }

            public void OnConnect(bool isConnect)
            {
                if (asReaderA24DDelegate.OnConnect != null)
                {
                    asReaderA24DDelegate.OnConnect(isConnect);
                }
            }

            public void OnBatteryStateReceived(int battery)
            {
                if (asReaderA24DDelegate.OnBatteryStateReceived != null)
                {
                    asReaderA24DDelegate.OnBatteryStateReceived(battery);
                }
            }

            public void OnReaderInfoReceived(IDictionary<string, string> info)
            {
                if (asReaderA24DDelegate.OnReaderInfoReceived != null)
                {
                    Dictionary<string, string> dic = new Dictionary<string, string>();
                    foreach (string key in info.Keys)
                    {
                        dic[key] = info[key];
                    }
                    asReaderA24DDelegate.OnReaderInfoReceived(dic);
                }
            }

            public void OnDeviceManagerError(AsReaderError.ErrorCode errorCode)
            {
                if (asReaderA24DDelegate.OnDeviceManagerError != null)
                {
                    AsReaderA24D_ErrorCode code = (AsReaderA24D_ErrorCode)Convert.ToInt32(errorCode);
                    asReaderA24DDelegate.OnDeviceManagerError(code);
                }
            }

            public void OnTriggerEventReceived(DeviceConst.DeviceTriggerEvent triggerEvent)
            {
               
            }
        }

        private MyAsReaderDeviceDelegate myAsReaderDeviceDelegate;
        private BarcodeAsReaderDeviceDelegate barcodeAsReaderDeviceDelegate;
        static AsReaderA24DManager uniqueInstance;
        static readonly object locker = new object();
        public static AsReaderA24DManager sharedInstance()
        {
            if (uniqueInstance == null)
            {
                lock (locker)
                {
                    if (uniqueInstance == null)
                    {
                        uniqueInstance = new AsReaderA24DManager();
                    }
                }
            }
            return uniqueInstance;
        }
        public AsReaderA24DManager()
        {
            myAsReaderDeviceDelegate = new MyAsReaderDeviceDelegate();
            AsReader.Instance.DeviceManager.SetCallback(myAsReaderDeviceDelegate);
            barcodeAsReaderDeviceDelegate = new BarcodeAsReaderDeviceDelegate();
            AsReader.Instance.BarcodeManager.SetCallback(barcodeAsReaderDeviceDelegate);
        }
        public void Init()
        {
            AsReader.Instance.DeviceManager.SetPackageName(MainActivity.instance.PackageName);
            AsReader.Instance.Initialize(MainActivity.instance, ConnectionType.Usb);
        }
        public string GetAPPVersion()
        {
            var activity = MainActivity.instance;
            return activity.PackageManager.GetPackageInfo(activity.PackageName, 0).VersionName;
        }
        public string GetSDKVersion()
        {
            if (GetAsReaderStatus() == false)
            {
                return "";
            }
            else
            {
                return AsReader.Instance.Version;
            }
        }
        public void GetReaderInfo()
        {
            if (GetAsReaderStatus())
            {
                AsReader.Instance.DeviceManager.GetReaderInfo();
            }
        }
        public void StartDecode()
        {
            if (GetAsReaderStatus() == true)
            {
                AsReader.Instance.BarcodeManager.StartDecode();
            }
        }
        public void StopDecode()
        {
            if (GetAsReaderStatus() == true)
            {
                AsReader.Instance.BarcodeManager.StopDecode();
            }
        }
        public void SetDeviceDefault()
        {
            if (GetAsReaderStatus() == true)
            {
                string beep = Preferences.Get("beep", "true");
                bool beepStatus;
                if (beep == "true")
                {
                    beepStatus = true;
                }
                else
                {
                    beepStatus = false;
                }
                string vibration = Preferences.Get("vibration", "true");
                bool vibrationStatus;
                if (vibration == "true")
                {
                    vibrationStatus = true;
                }
                else
                {
                    vibrationStatus = false;
                }
                AsReader.Instance.DeviceManager.SetReaderSettings(beepStatus, vibrationStatus, true, true, true);
            }
        }
        public bool GetAsReaderStatus()
        {
            return AsReader.Instance.DeviceManager.IsConnected;
        }
        public string GetAsReaderA24DSDKDLL()
        {
            return AsReaderA24DSDKDLL.DLLVersion();
        }
    }
}
